<?php
namespace Drupal\email_contact\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\email_contact\Form\ContactForm;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Core\Entity\ContentEntityBase;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class ContactController.
 *
 * @package Drupal\email_contact\Controller
*/
class ContactController extends ControllerBase {

  /**
  * A logger instance.
  *
  * @var \Psr\Log\LoggerInterface
  */
  protected $logger;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
   protected $entityTypeManager;

  /**
   * Constructs an ContactController instance.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
   public function __construct(LoggerInterface $logger, EntityTypeManagerInterface $entity_type_manager) {
    $this->logger = $logger;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
  * {@inheritdoc}
  */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('logger.factory')->get('action'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getTitle($entity_type = NULL, $entity_id = NULL, $field_name = NULL, $view_mode = 'full') {
    if ($entity_id == NULL) {
      return '';
     }
     
    $entity = \Drupal::entityTypeManager()->getStorage($entity_type)->load($entity_id);
    $title = method_exists($entity, 'getTitle') ? $entity->getTitle() . '  - Email Contact' : 'Email Contact';
    return $title;
  }

  /**
   * {@inheritdoc}
   */
  public function content($entity_type = NULL, $entity_id = NULL, $field_name = NULL, $view_mode = 'full') {
    if (!in_array($entity_type, array_keys($this->entityTypeManager->getDefinitions()))) {
      throw new NotFoundHttpException();
    }
    $entity = $this->entityTypeManager->getStorage($entity_type)->load($entity_id);
    if (!$entity instanceof ContentEntityBase) {
      throw new NotFoundHttpException();
    }
    $view_display = EntityViewDisplay::collectRenderDisplay($entity, $view_mode);
    $field_display = $view_display->getComponent($field_name);
    $field_settings = NULL;
    if (isset($field_display['settings'])) {
      $field_settings = $field_display['settings'];
    }
    try {
      $form = new ContactForm($entity_type, $entity_id, $field_name, $field_settings);
      return \Drupal::formBuilder()->getForm($form);
    }
    catch (NotFoundHttpException $e) {
      $this->logger->notice('Invalid contact form on @entity_type id @id.', ['@entity_type' => $entity->getEntityTypeId(), '@id' => $entity->id()]);
      throw new NotFoundHttpException();
    }

  }
}
